@@ -5,13 +5,12 @@
 #include <PDFDoc.h>
 #include <PSOutputDev.h>
 #include <Link.h>
+#include <Form.h>
 #include <Gfx.h>
 #include <FontInfo.h>
 #include <TextOutputDev.h>
 #include <Catalog.h>
 
-#include "poppler-attachment.h"
-
 #if defined (HAVE_CAIRO)
 #include <CairoOutputDev.h>
 #elif defined (HAVE_SPLASH)
@@ -32,6 +31,8 @@ struct _PopplerDocument
 
 struct _PopplerPSFile
 {
+  GObject parent_instance;
+
   PopplerDocument *document;
   PSOutputDev *out;
   char *filename;
@@ -44,6 +45,7 @@ struct _PopplerPSFile
 
 struct _PopplerFontInfo
 {
+  GObject parent_instance;
   PopplerDocument *document;
   FontInfoScanner *scanner;
 };
@@ -56,18 +58,33 @@ struct _PopplerPage
   int index;
   TextOutputDev *text_dev;
   Gfx *gfx;
+#if defined (HAVE_CAIRO)
+  CairoImageOutputDev *image_dev;
+#endif
 };
 
+struct _PopplerFormField
+{
+  GObject parent_instance;
+  PopplerDocument *document;
+  FormWidget *widget;
+};
+
+
 PopplerPage   *_poppler_page_new   (PopplerDocument *document,
 				    Page            *page,
 				    int              index);
 PopplerAction *_poppler_action_new (PopplerDocument *document,
 				    LinkAction      *link,
 				    const gchar     *title);
-
+PopplerDest   *_poppler_dest_new_goto (PopplerDocument *document,
+				       LinkDest        *link_dest);
+PopplerFormField *_poppler_form_field_new (PopplerDocument *document,
+					   FormWidget      *field);
 PopplerAttachment *_poppler_attachment_new (PopplerDocument *document,
 					    EmbFile         *file);
 
 char *_poppler_goo_string_to_utf8(GooString *s);
-
+gboolean _poppler_convert_pdf_date_to_gtime (GooString *date,
+					     GTime     *gdate);
 #endif
