@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 // qt/kde includes
@@ -785,6 +785,7 @@ void LineAnnotation::store( QDomNode & node, QDomDocument & document ) const
             lineElement.appendChild( pElement );
             pElement.setAttribute( "x", p.x() );
             pElement.setAttribute( "y", p.y() );
+            ++it;
         }
     }
 }
