@@ -1,7 +1,6 @@
 #include <QtCore/QtCore>
 #include <iostream>
 
-#define UNSTABLE_POPPLER_QT4
 #include <poppler-qt4.h>
 
 int main( int argc, char **argv )
@@ -33,5 +32,6 @@ int main( int argc, char **argv )
     } else {
 	std::cout << "There are no embedded document at the top level" << std::endl;
     }
+    delete doc;
   
 }
