@@ -14,6 +14,8 @@
 
 #include <string.h>
 #include <t1lib.h>
+#include "goo/GooString.h"
+#include "goo/gfile.h"
 #include "goo/gmem.h"
 #include "SplashT1FontEngine.h"
 #include "SplashT1Font.h"
@@ -25,8 +27,7 @@
 
 SplashFontFile *SplashT1FontFile::loadType1Font(SplashT1FontEngine *engineA,
 						SplashFontFileID *idA,
-						char *fileNameA,
-						GBool deleteFileA,
+						SplashFontSrc *src,
 						char **encA) {
   int t1libIDA;
   char **encTmp;
@@ -35,8 +36,26 @@ SplashFontFile *SplashT1FontFile::loadType1Font(SplashT1FontEngine *engineA,
   char *encPtr;
   int i;
 
+  GooString *fileNameA;
+  SplashFontSrc *newsrc = NULL;
+  SplashFontFile *ff;
+
+  if (! src->isFile) {
+    GooString *tmpFileName;
+    FILE *tmpFile;
+    if (!openTempFile(&tmpFileName, &tmpFile, "wb", NULL))
+      return NULL;
+    fwrite(src->buf, 1, src->bufLen, tmpFile);
+    fclose(tmpFile);
+    newsrc = new SplashFontSrc;
+    newsrc->setFile(tmpFileName, gTrue);
+    src = newsrc;
+    delete tmpFileName;
+  }
+  fileNameA = src->fileName;
   // load the font file
-  if ((t1libIDA = T1_AddFont(fileNameA)) < 0) {
+  if ((t1libIDA = T1_AddFont(fileNameA->getCString())) < 0) {
+    delete newsrc;
     return NULL;
   }
   T1_LoadFont(t1libIDA);
@@ -63,15 +82,18 @@ SplashFontFile *SplashT1FontFile::loadType1Font(SplashT1FontEngine *engineA,
   encTmp[256] = "custom";
   T1_ReencodeFont(t1libIDA, encTmp);
 
-  return new SplashT1FontFile(engineA, idA, fileNameA, deleteFileA,
+  ff = new SplashT1FontFile(engineA, idA, src,
 			      t1libIDA, encTmp, encStrTmp);
+  if (newsrc)
+    newsrc->unref();
+  return ff;
 }
 
 SplashT1FontFile::SplashT1FontFile(SplashT1FontEngine *engineA,
 				   SplashFontFileID *idA,
-				   char *fileNameA, GBool deleteFileA,
+				   SplashFontSrc *srcA,
 				   int t1libIDA, char **encA, char *encStrA):
-  SplashFontFile(idA, fileNameA, deleteFileA)
+  SplashFontFile(idA, srcA)
 {
   engine = engineA;
   t1libID = t1libIDA;
@@ -85,10 +107,11 @@ SplashT1FontFile::~SplashT1FontFile() {
   T1_DeleteFont(t1libID);
 }
 
-SplashFont *SplashT1FontFile::makeFont(SplashCoord *mat) {
+SplashFont *SplashT1FontFile::makeFont(SplashCoord *mat,
+				       SplashCoord *textMat) {
   SplashFont *font;
 
-  font = new SplashT1Font(this, mat);
+  font = new SplashT1Font(this, mat, textMat);
   font->initCache();
   return font;
 }
